package com.elitesland.system.entity;

import com.elitesland.core.base.BaseModel;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

/**
 * <pre>
 * [功能说明]
 * </pre>
 *
 * @author Mir
 * @date 2020/6/15
 */
@Entity
@Table(name = "sys_permission")
@org.hibernate.annotations.Table(appliesTo = "sys_permission", comment = "系统权限表，包括URL、操作")
@Data
@Accessors(chain = true)
@ApiModel(value = "系统权限表", description = "系统权限表，包括URL、操作")
@DynamicUpdate
public class SysPermissionDO extends BaseModel implements Serializable {
    private static final long serialVersionUID = 6810626977765149177L;

    @ApiModelProperty(value = "权限代码")
    @Column(columnDefinition = "varchar(64) default '' comment '权限代码'", nullable = false, unique = true)
    @NotBlank(message = "权限代码不能为空")
    private String code;

    @ApiModelProperty(value = "权限名称")
    @Column(columnDefinition = "varchar(16) default '' comment '权限名称'", nullable = false)
    @NotBlank(message = "权限名称不能为空")
    private String name;

    @ApiModelProperty(value = "权限类型")
    @Column(columnDefinition = "int default 0 comment '权限类型'")
    private Integer permType;

    @ApiModelProperty(value = "父权限ID")
    @Column(columnDefinition = "bigint default 0 comment '父权限ID'")
    private Long parentId;

    @ApiModelProperty(value = "权限层级路径")
    @Column(columnDefinition = "varchar(256) default '' comment '权限层级路径'")
    private String path;

    @ApiModelProperty(value = "权限层级")
    @Column(columnDefinition = "int default 0 comment '权限层级'")
    private Integer level;

    @ApiModelProperty("排列顺序")
    @Column(columnDefinition = "int default 0 comment '排列顺序'")
    private Integer sortNo;

    @ApiModelProperty(value = "URL路径模式")
    @Column(columnDefinition = "varchar(256) default '' comment 'URL路径模式'")
    @NotBlank(message = "URL路径不能为空")
    private String pattern;

    @ApiModelProperty(value = "URL的HTTP方法[COM:HTTP_METH]")
    @Column(columnDefinition = "varchar(32) default '' comment 'URL的HTTP方法[COM:HTTP_METH]'")
    String httpMethod;

    @ManyToMany(mappedBy = "permissions", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JsonIgnoreProperties({"permissions"})
    private Set<SysRoleDO> roles;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SysPermissionDO)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        SysPermissionDO entity = (SysPermissionDO) o;
        return getId().equals(entity.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }
}