package com.elitesland.system.entity;

import com.elitesland.core.base.BaseModel;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

/**
 * <pre>
 * [功能说明]
 * </pre>
 *
 * @author Mir
 * @date 2020/6/15
 */
@Entity
@Table(name = "sys_role")
@org.hibernate.annotations.Table(appliesTo = "sys_role", comment = "系统角色表")
@Data
@Accessors(chain = true)
@ApiModel(value = "系统角色表", description = "系统角色表")
@DynamicUpdate
public class SysRoleDO extends BaseModel implements Serializable {
    private static final long serialVersionUID = 5152159757116510744L;

    @ApiModelProperty(value = "角色代码")
    @Column(columnDefinition = "varchar(16) default '' comment '角色代码'", nullable = false, unique = true)
    private String code;

    @ApiModelProperty(value = "角色名称")
    @Column(columnDefinition = "varchar(32) default '' comment '角色名称'", nullable = false)
    private String name;

    @ApiModelProperty(value = "是否启用")
    @Column(columnDefinition = "tinyint default 1 comment '是否启用'")
    Boolean enabled;

    @ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JoinTable(name = "sys_role_perm", joinColumns = @JoinColumn(name = "role_id"), inverseJoinColumns = @JoinColumn(name = "permission_id"))
    @JsonIgnoreProperties({"roles"})
    private Set<SysPermissionDO> permissions;

    @ManyToMany(mappedBy = "roles", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JsonIgnoreProperties({"roles"})
    private Set<SysUserDO> users;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SysRoleDO)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        SysRoleDO entity = (SysRoleDO) o;
        return getId().equals(entity.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }
}
