package com.elitesland.system.entity;

import com.elitesland.core.base.BaseModel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * <p>
 * 分类码明细表
 * </p>
 *
 * @author Mir
 * @date 2020/5/15
 */
@Entity
@Table(name = "sys_udc")
@org.hibernate.annotations.Table(appliesTo = "sys_udc", comment = "分类码表")
@Data
@Accessors(chain = true)
@ApiModel(value = "记录分类码的系统表", description = "分类码表")
public class SysUdcDO extends BaseModel implements Serializable {

    private static final long serialVersionUID = -6618541083363538072L;

    @ApiModelProperty(value = "领域码")
    @Column(columnDefinition = "varchar(32)  default '' comment '领域码'", nullable = false)
    String domainCode;

    @ApiModelProperty(value = "UDC码")
    @Column(columnDefinition = "varchar(32) default '' comment 'UDC码'", nullable = false)
    String udcCode;

    @ApiModelProperty(value = "UDC码名称")
    @Column(columnDefinition = "varchar(64) default '' comment 'UDC名称'")
    String udcName;

    @ApiModelProperty(value = "UDC值")
    @Column(columnDefinition = "varchar(32) default '' comment 'UDC值'", nullable = false)
    String udcVal;

    @ApiModelProperty(value = "值描述")
    @Column(columnDefinition = "varchar(64) default '' comment '值描述'")
    String valDesc;

    @ApiModelProperty(value = "排序号")
    @Column(columnDefinition = "int default 0 comment '排序号'")
    Integer valSortNo;

    @ApiModelProperty(value = "硬编码")
    @Column(columnDefinition = "tinyint default 0 comment '是否硬编码'")
    Boolean hdFlag;

    @ApiModelProperty(value = "是否开启数据权限")
    @Column(columnDefinition = "tinyint default 0 comment '是否开启数据权限'")
    Boolean dpFlag;

    @ApiModelProperty(value = "扩展字段1-特殊处理")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段1'")
    String es1;

    @ApiModelProperty(value = "扩展字段2")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段2'")
    String es2;

    @ApiModelProperty(value = "扩展字段3")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段3'")
    String es3;

    @ApiModelProperty(value = "扩展字段4")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段4'")
    String es4;

    @ApiModelProperty(value = "扩展字段5")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段5'")
    String es5;

    @ApiModelProperty(value = "扩展字段6")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段6'")
    String es6;

    @ApiModelProperty(value = "扩展字段7")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段7'")
    String es7;

    @ApiModelProperty(value = "扩展字段8")
    @Column(columnDefinition = "varchar(32) default '' comment '扩展字段8'")
    String es8;

    @ApiModelProperty(value = "扩展浮点1")
    @Column(columnDefinition = "numeric(18,4) default 0 comment '扩展浮点1'")
    Double ef1;

    @ApiModelProperty(value = "扩展浮点3")
    @Column(columnDefinition = "numeric(18,4) default 0 comment '扩展浮点2'")
    Double ef2;

    @ApiModelProperty(value = "扩展浮点3")
    @Column(columnDefinition = "numeric(18,4) default 0 comment '扩展浮点3'")
    Double ef3;

    @ApiModelProperty(value = "扩展浮点4")
    @Column(columnDefinition = "numeric(18,4) default 0 comment '扩展浮点4'")
    Double ef4;

    @ApiModelProperty(value = "扩展整数1")
    @Column(columnDefinition = "int default 0 comment '扩展整数1'")
    Integer ei1;

    @ApiModelProperty(value = "扩展整数2")
    @Column(columnDefinition = "int default 0 comment '扩展整数2'")
    Integer ei2;

    @ApiModelProperty(value = "扩展整数3")
    @Column(columnDefinition = "int default 0 comment '扩展整数3'")
    Integer ei3;

    @ApiModelProperty(value = "扩展整数4")
    @Column(columnDefinition = "int default 0 comment '扩展整数4'")
    Integer ei4;

    @ApiModelProperty(value = "扩展日期1")
    @Column(columnDefinition = "datetime default NULL comment '扩展日期1'")
    LocalDateTime ed1;

    @ApiModelProperty(value = "扩展日期2")
    @Column(columnDefinition = "datetime default NULL comment '扩展日期2'")
    LocalDateTime ed2;

    @ApiModelProperty(value = "扩展日期3")
    @Column(columnDefinition = "datetime default NULL comment '扩展日期3'")
    LocalDateTime ed3;

    @ApiModelProperty(value = "扩展日期4")
    @Column(columnDefinition = "datetime default NULL comment '扩展日期4'")
    LocalDateTime ed4;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SysUdcDO)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        SysUdcDO entity = (SysUdcDO) o;
        return getId().equals(entity.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }
}
