package com.elitesland.system.entity;

import com.elitesland.core.base.BaseModel;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

/**
 * <pre>
 *     系统用户账号表
 * </pre>
 *
 * @author Mir
 * @date 2020-06-15
 */
@Entity
@Table(name = "sys_user")
@org.hibernate.annotations.Table(appliesTo = "sys_user", comment = "系统用户账号表")
@Data
@Accessors(chain = true)
@ApiModel(value = "系统用户账号表", description = "系统用户账号表")
@DynamicUpdate
public class SysUserDO extends BaseModel implements Serializable {

    private static final long serialVersionUID = -7840804475330315230L;

    @ApiModelProperty(value = "用户账号名称")
    @Column(columnDefinition = "varchar(32) default '' comment '用户账号名称'", nullable = false, unique = true)
    private String username;

    @ApiModelProperty(value = "用户姓氏")
    @Column(columnDefinition = "varchar(16) default '' comment '用户姓氏'")
    private String lastName;

    @ApiModelProperty(value = "用户名称")
    @Column(columnDefinition = "varchar(16) default '' comment '用户名称'")
    private String firstName;

    @ApiModelProperty(value = "用户手机号码")
    @Column(columnDefinition = "varchar(16) default '' comment '用户手机号码'", unique = true)
    private String mobile;

    @ApiModelProperty(value = "用户邮箱地址")
    @Column(columnDefinition = "varchar(32) default '' comment '用户邮箱地址'")
    private String email;

    @ApiModelProperty(value = "用户账号密码-hashed")
    @Column(columnDefinition = "varchar(128) default '' comment '用户账号密码-hashed'", nullable = false)
    private String password;

    @ApiModelProperty(value = "是否启动")
    @Column(columnDefinition = "tinyint default 1 comment '是否启动'")
    private Boolean enabled;

    @ManyToMany(cascade = CascadeType.PERSIST, fetch = FetchType.EAGER)
    @JoinTable(name = "sys_user_role", joinColumns = @JoinColumn(name = "user_id"), inverseJoinColumns = @JoinColumn(name = "role_id"))
    @JsonIgnoreProperties({"users"})
    private Set<SysRoleDO> roles;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SysUserDO)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        SysUserDO entity = (SysUserDO) o;
        return getId().equals(entity.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }
}